#-- encoding: UTF-8

#-- copyright
# OpenProject is a project management system.
# Copyright (C) 2012-2018 the OpenProject Foundation (OPF)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 3.
#
# OpenProject is a fork of ChiliProject, which is a fork of Redmine. The copyright follows:
# Copyright (C) 2006-2017 Jean-Philippe Lang
# Copyright (C) 2010-2013 the ChiliProject Team
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# See docs/COPYRIGHT.rdoc for more details.
#++

class Queries::WorkPackages::Filter::StatusFilter < Queries::WorkPackages::Filter::WorkPackageFilter
  def allowed_values
    all_statuses.map { |s| [s.name, s.id.to_s] }
  end

  def available_operators
    [Queries::Operators::OpenWorkPackages,
     Queries::Operators::Equals,
     Queries::Operators::ClosedWorkPackages,
     Queries::Operators::NotEquals,
     Queries::Operators::All]
  end

  def available?
    all_statuses.any?
  end

  def type
    :list
  end

  def self.key
    :status_id
  end

  def value_objects
    values_ids = values.map(&:to_i)

    all_statuses.select { |status| values_ids.include?(status.id) }
  end

  def allowed_objects
    all_statuses
  end

  def ar_object_filter?
    true
  end

  private

  def all_statuses
    key = 'Queries::WorkPackages::Filter::StatusFilter/all_statuses'

    RequestStore.fetch(key) do
      Status.all.to_a
    end
  end

  def operator_strategy
    super_value = super

    if !super_value
      case operator
      when 'o'
        Queries::Operators::OpenWorkPackages
      when 'c'
        Queries::Operators::ClosedWorkPackages
      when '*'
        Queries::Operators::All
      end
    else
      super_value
    end
  end
end
