#-- copyright
# OpenProject is a project management system.
# Copyright (C) 2012-2018 the OpenProject Foundation (OPF)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 3.
#
# OpenProject is a fork of ChiliProject, which is a fork of Redmine. The copyright follows:
# Copyright (C) 2006-2017 Jean-Philippe Lang
# Copyright (C) 2010-2013 the ChiliProject Team
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# See docs/COPYRIGHT.rdoc for more details.
#++

module API
  class OpenProjectAPI < ::Grape::API
    class << self
      def inherited(api, *)
        super

        # run unscoped patches (i.e. patches that are on the class root, not in a namespace)
        api.apply_patches(nil)
      end
    end
  end
end

Grape::DSL::Routing::ClassMethods.module_eval do
  # Be reload safe. otherwise, an infinite loop occurs on reload.
  unless instance_methods.include?(:orig_namespace)
    alias :orig_namespace :namespace
  end

  def namespace(space = nil, options = {}, &block)
    orig_namespace(space, options) do
      instance_eval(&block)
      apply_patches(space)
    end
  end

  def apply_patches(path)
    (patches[path] || []).each do |patch|
      instance_eval(&patch)
    end
  end

  def patches
    ::Constants::APIPatchRegistry.patches_for(base)
  end
end
