#-- copyright
# OpenProject is a project management system.
# Copyright (C) 2012-2018 the OpenProject Foundation (OPF)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 3.
#
# OpenProject is a fork of ChiliProject, which is a fork of Redmine. The copyright follows:
# Copyright (C) 2006-2017 Jean-Philippe Lang
# Copyright (C) 2010-2013 the ChiliProject Team
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# See docs/COPYRIGHT.rdoc for more details.
#++

require 'spec_helper'

describe 'OpenProject include wiki page macro' do
  include ActionView::Helpers::UrlHelper
  include OpenProject::StaticRouting::UrlHelpers
  include OpenProject::TextFormatting

  def controller
    # no-op
  end

  let(:project) do
    FactoryBot.build_stubbed :project
  end

  let(:input) {}
  subject { format_text(input, project: project) }

  def error_html(exception_msg)
    "<p><macro class=\"macro-unavailable\" data-macro-name=\"include_wiki_page\">" \
          "Error executing the macro include_wiki_page (#{exception_msg})</macro></p>"
  end

  context 'old macro syntax no longer works' do
    let(:input) { '{{include(whatever)}}' }
    it { is_expected.to be_html_eql("<p>#{input}</p>") }
  end

  context 'with the new but also no longer supported syntax' do
    let(:input) { '<macro class="include_wiki_page" data-page="included"></macro>' }
    it { is_expected.to be_html_eql(error_html("The macro does no longer exist.")) }
  end
end
